module DkComposerTasks
  extend ActiveSupport::Concern
  module ClassMethods
  end

  included do
    desc 'lsa [KEYWORD]  [-I loadpath] [-s] [-c] ', <<-LONG
    'list all applications which app name match the patten'
    example:
      dk lsa                    #list all apps
      dk lsa test               #search all apps which's name include string test;
      dk lsa -I somepath        #list all apps include with your specific path
    LONG
    method_option :loadpaths, type: :array, aliases: 'I', desc: 'loadpath'
    method_option :source, type: :boolean, aliases: 's', desc: 'show as source format'
    method_option :compose, type: :boolean, aliases: 'c', desc: 'show as compose format'

    def lsa(keyword = nil)
      auto_load(options[:loadpaths])
      ret = DkComposer::STACK.select { |_k, v| /#{keyword}/ =~ v.name.to_s }

      if options[:source]
        ret.each do |k, v|
          puts ["#-------------stack:#{k}--------------",
                v.dump,
                ''].join("\n")
        end

        return ret
      end

      if options[:compose]
        ret.each do |k, v|
          puts ["#-------------stack:#{k}--------------",
                v.to_yaml,
                ''].join("\n")
        end

        return ret
      end
      print_table ret.to_table(:name, :shortdesc)
      ret
    end

    desc 'lsi [KEYWORD]  [-I loadpath] ', <<-LONG
    'list all images which  name match the patten'
    example:
      dk lsi                    #list all images
      dk lsi test               #search all images which's name include string test;
      dk lsi test -f            #search all images which's name include string test and only output dockerfile;
      dk lsi -I somepath        #list all images include with your specific path
    LONG
    method_option :loadpaths, type: :array, aliases: 'I'
    method_option :dockerfile, type: :boolean, aliases: 'f'

    def lsi(keyword = nil)
      auto_load(options[:loadpaths])
      ret = DkComposer::IMAGE.select { |k, v| /#{keyword}/ =~ k.to_s && v && /:latest$/ !~ k.to_s }
      if options[:dockerfile]

        ret.each do |k, v|
          puts ["#-------------image:#{k}--------------",
                v.to_docker_file,
                ''].join("\n")
        end
      else
        print_table ret.to_table(:name, :tag, :from, :use, :repos, :shortdesc)
      end
      ret
    end

    desc 'image [name] [build,pull,push]', <<-LONG
    'image actions'
    example:
      dk image                    #list all apps and ask one for image
      dk image test param1 param2 #list all apps which's name include string test and ask one for image and pass the params in;
      dk image -I somepath        #list all apps and ask one for image and include with your specific path
    LONG
    method_option :loadpaths, type: :array, aliases: 'I'

    def image(*params)
      name, action, = params
      auto_load(options[:loadpaths])
      ret = DkComposer::IMAGE.select { |k, v| /#{name}/ =~ k.to_s && v && /:latest$/ !~ k.to_s }
      return say('There is no image avaliable'.red) if ret.empty?
      img = if ret.count > 1
              ask_and_select(ret, :name, :tag, :from, :use, :repos, :shortdesc)
            else
              print_table ret.to_table(:name, :tag, :from, :use, :repos, :shortdesc)
              ret.to_a.first.last
            end
      return unless img
      actions = %w[pull build push]
      action = ask('pls input a correct action to the image', limited_to: actions.unshift('q')) if action.nil? || !actions.include?(action.to_s)
      return if action == 'q'
      say(img.send(action.to_sym))
    end

    desc 'up [KEYWORD]', <<-LONG
    'up application which app name match the patten'
    example:
      dk up                    #list all apps and ask one for up
      dk up test param1 param2 #list all apps which's name include string test and ask one for up and pass the params in;
      dk up -I somepath        #list all apps and ask one for up and include with your specific path
    LONG
    method_option :loadpaths, type: :array, aliases: 'I'

    def up(*params)
      name, params = params
      if name && name.start_with?('-')
        params ||= []
        params.unshift name
        name = nil
      end
      auto_load(options[:loadpaths])
      ret = DkComposer::STACK.select { |_k, v| /#{name}/ =~ v.name.to_s }
      return say('There is no app avaliable'.red) if ret.empty?
      app = if ret.count > 1
              ask_and_select(ret)
            else
              ret.to_a.first.last
            end
      return unless app
      app.run *params
    end

  desc 'down [KEYWORD]', <<-LONG
    'down application which app name match the patten'
    example:
      dk down                    #list all apps and ask one for down
      dk down test param1 param2 #list all apps which's name include string test and ask one for down and pass the params in;
      dk down -I somepath        #list all apps and ask one for down and include with your specific path
    LONG
    method_option :loadpaths, type: :array, aliases: 'I'

    def down(*params)
      name, params = params
      if name && name.start_with?('-')
        params ||= []
        params.unshift name
        name = nil
      end
      auto_load(options[:loadpaths])
      ret = DkComposer::STACK.select { |_k, v| /#{name}/ =~ v.name.to_s }
      return say('There is no app avaliable'.red) if ret.empty?
      app = if ret.count > 1
              ask_and_select(ret)
            else
              ret.to_a.first.last
            end
      return unless app
      app.stop *params
    end

    desc 'create appname', <<-LONG
    'create application with name'
    example:
      dk create test                    #create an app with name 'test'
    LONG
    def create(name)
      auto_load(options[:loadpaths])
      @opt = { name: name }
      answer = ask('Create stack from exist stack? yes=from exists, no=create new stack,q=quit', limited_to: %w[q y n])
      return if answer == 'q'
      target_file = "#{name}.dk.rb"
      if answer == 'y'
        ret = DkComposer::STACK
        print_table ret.to_table(:name, :shortdesc, :longdesc)
        answer = ask('input the stack name', limited_to: ret.keys.map(&:to_s))
        stack = ret[answer.to_sym]
        stack.build_path = '.'

        create_file(target_file, stack.to_s)
      end
      if answer == 'n'
        ret = DkComposer::SERVICE
        print_table ret.to_table(:name, :shortdesc, :longdesc)
        answers = multi_ask('Add a service into the app,input the service name', limited_to: ret.keys.map(&:to_s).unshift(''))
        @opt[:services] = answers.uniq
        template("#{self.class.source_root}/templates/stack.dk.rb", target_file)
      end
      beautify_file(target_file)
    end

    private

    def beautify_file(file)
      result = run('which rubocop', capture: true)
      run("rubocop -a #{file}", capture: true) unless result.empty?
    end

    def ask_and_select(h, *cols)
      cols = %i[name shortdesc] if cols.empty?
      print_table h.to_table(*cols, with_index: true)
      say('which app you want to up?'.blue)
      answer = ask('(q=QUIT;n=)', limited_to: ['q', *('1'..h.count.to_s).to_a])
      return nil if answer == 'q'
      h.to_a[answer.to_i - 1].last
    end

    def auto_load(loadpaths = [])
      loadpaths ||= []
      loadpaths.unshift(*(ENV['DK_LOAD_PATH'] || '').split(':')).uniq!
      loadpaths.push '.'
      loadpaths.each do |path|
        $LOAD_PATH.unshift(path) unless $LOAD_PATH.include?(path)
        dk_pattern = "#{path}/*.dk.rb"
        Dir[dk_pattern].each { |dkfile| require dkfile }
      end
    end
  end
end
