module ImageTasks
  extend ActiveSupport::Concern
  module ClassMethods
  end

  included do
    desc 'ls [KEYWORD]  [-n nk] [-t tk]', <<-LONG
    'list all images which image_name match the patten'
    example:
      dk ls                    #list all images
      dk ls test               #search all images which include string test;
      dk ls test -n nk      #search all images which include test and name include nk;
      dk ls test -t tk       #search all images include test and tag include tk;
      dk ls -n nk -t tk   #search all images which repository include nk and tag include tk
    LONG
    method_option :name, type: :string, aliases: 'n'
    method_option :tag, type: :string, aliases: 't'
    def ls(keyword = nil)
      ret = Image.find(keyword: keyword, name: options['name'], tag: options['tag'])
      Image.print_table(ret)
      #      print_table([ret.unshift(Image.headers)])
    end

    desc 'rmi [KEYWORD] [-n nk] [-t tk]', <<-LONG
    'delete images which image_name match the patten'
    example:
      dk rmi                    #delete all images
      dk rmi test               #delete all images which include string test;
      dk rmi test -n nk         #delete all images which include test and name include nk;
      dk rmi test -t tk         #delete all images which include test and tag include tk;
      dk rmi -n nk -t tk        #delete all images which repository include nk and tag include tk
    LONG
    method_option :name, type: :string, aliases: 'n'
    method_option :tag, type: :string, aliases: 't'
    def rmi(keyword = nil)
      ret = Image.find(keyword: keyword, name: options['name'], tag: options['tag'])
      Image.print_table(ret)
      say('The listed images will be deleted, ARE YOU SURE?'.blue)
      answer = ask('(y=YES;q=QUIT;i=ASK one by one)', limited_to: %w[y q i])
      return if answer == 'q'
      ret.each do |image|
        if answer == 'i'
          say(image.to_s)
          each_answer = ask('going to delete? (y=YES,n=NO,q=QUIT):')
          next if each_answer == 'n'
          break if each_answer == 'q'
        end
        image.del
      end
    end

    desc 'pull [IMAGENAME] [-as one two three]', <<-LONG
     'pull ruby:2.3.4 -l myruby ruby:2.3'
    LONG
    method_option :local, type: :array, aliases: 'l'
    def pull(name)
      Image.pull(name)
      if options['local']
        options['local'].each do |new_name|
          Image.tag(name, new_name)
        end
      end
    end

    desc 'push [KEYWORD] [-n nk] [-t tk]', <<-LONG
    'select images which image_name match the patten and select some to push'
    example:
      dk push                    #list all images and select some to push
      dk push test               #list all images which include string test and select some to push;
      dk push test -n nk         #list all images which include test and name include nk and select some to push;
      dk push test -t tk         #list all images which include test and tag include tk and select some to push;
      dk push -n nk -t tk        #list all images which repository include nk and tag include tk and select some to push;
    LONG
    method_option :name, type: :string, aliases: 'n'
    method_option :tag, type: :string, aliases: 't'
    def push(keyword = nil)
      ret = Image.find(keyword: keyword, name: options['name'], tag: options['tag'])
      Image.print_table(ret)
      say('The listed images will be pushed, ARE YOU SURE?'.blue)
      answer = ask('(y=YES;q=QUIT;i=ASK one by one)', limited_to: %w[y q i])
      return if answer == 'q'
      ret.each do |image|
        if answer == 'i'
          say(image.to_s)
          each_answer = ask('going to push? (y=YES,n=NO,c=YES and change repository name,q=QUIT):')
          next if each_answer == 'n'
          break if each_answer == 'q'
          if each_answer == 'c'
            new_rep = ask_with_default('New repository?', image.image_name)
            Image.tag(image.image_name, new_rep)
            image.repository, image.tag = new_rep.split(':')
          end
        end
        image.push
      end
    end

    desc 'tag [IMAGENAME] [-t one two three]', <<-LONG
     tag image as new images

     samples:

     tag ruby:2.3.4 -t a:2.3.4 haha:2.3.4
    LONG
    method_option :as, type: :array
    def tag(name)
      image = search_with_ask(name, Image, 'which image you want to tag?')
      say(image.to_s)
      option ||= {}
      new_image_name = option['as'] || ask('new image name?')
      Image.tag(image.image_name, new_image_name)
    end

    desc 'xrun [KEYWORD] [-n nk] [-t tk] [--opts] [--cmd] [-s]', <<-LONG
    select images which image_name match the patten and select some to run
    example:
      dk xrun\
      #list all images and select some to run

      dk xrun test\
      #list all images which include string test and select one to run;

      dk xrun test -n nk\
      #list all images which include test and name include nk and select one to run;

      dk xrun test -t tk\
      #list all images which include test and tag include tk and select one to xrun;

      dk xrun -n nk -t tk\
      #list all images which repository include nk and tag include tk and select one to push;

      dk xrun --opts "'-i -t --name test'" --cmd '/bin/bash'\
      #NOTE  opts and cmd option ,use "''" to quot as sample
      #list all images and run as opts -it --name test and command is /bin/bash

      dk xrun --opts "'-i -t --name test'" --cmd '/bin/bash' -s test.bash\
      #NOTE  opts and cmd option ,use "''" to quot as sample
      #list all images and run as opts -it --name test and command is /bin/bash
      #and save the command as alias and alias name is test.bash
    LONG
    method_option :opts, type: :string, aliases: 'o'
    method_option :cmd, type: :string, aliases: 'c'
    method_option :name, type: :string, aliases: 'n'
    method_option :tag, type: :string, aliases: 't'
    method_option :alias, type: :string, aliases: 's'

    def xrun(keyword = nil)
      ret = Image.find(keyword: keyword, name: options['name'], tag: options['tag'])
      if ret.count==0
        repository,tag=keyword.split(":")
        ret=[Image.new(repository:repository,tag:tag)]
      end

      if ret.count > 1
        Image.print_table(ret) { |item, index| "[#{index + 1}]#{item}" }
        question = ['please select one image to run:'].unshift(' ').unshift(' ').join("\n")
        say(question.blue)
        answer = ask('(q=QUIT;n=)', limited_to: ['q', *('1'..ret.count.to_s).to_a])
        return if answer == 'q'
      end


      command = options['cmd'] || ask_with_default('Any command you want to run?', '')
      opts = options['opts'] || ask_with_default('Any option you want to set?', '-d')
      index=(answer&.to_i||1)-1
      xcommand=ret[index].launch(opts, command)
      run(["dk","alias",options['alias'],xcommand].join(" ")) if options['alias']
      run(xcommand.join(" "))
    end

    desc 'build [-d ./docker]', <<-LONG
    build docker file

    sample:

    dk build -d ./docker
    LONG
    method_option :dir, type: :string, aliases: 'd', default: '.'
    def build
      dir = options['dir']
      invoke(:init) if file_not_exist?("#{dir}/#{DOCKER_CONFIG_FILE}")
      load_config(dir) do |config|
          Image.build(DOCKER_FILE, *config[:repository_urls].map { |url| "#{url}/#{config[:image_name]}" }.unshift(config[:image_name]))
      end
    end

    desc 'release [-d ./docker]', <<-LONG
    release docker file

    sample:

    dk release -d ./docker
    LONG
    method_option :dir, type: :string, aliases: 'd', default: '.'
    def release
      dir = options['dir']
      invoke(:init) if file_not_exist?("#{dir}/#{DOCKER_CONFIG_FILE}")
      load_config(dir) do |config|
        config[:repository_urls].map { |url| "#{url}/#{config[:image_name]}" }.each do |image_name|
          Image.push(image_name)
        end
      end
    end
  end
end
